@@ -18,11 +18,9 @@ class FunctionalFuncActor(FunctionalActor):
     async def assign(self, *args, **kwargs):
         raise NotImplementedError("Please provide a func or overwrite the assign method!")
 
-    async def _assign(self, assign_handler: AssignHandler, reserve_handler: ReserveHandler, args, kwargs):
-        
-
+    async def _assign(self, assign_handler: AssignHandler, args, kwargs):
+    
         assign_handler_context.set(assign_handler)
-        reserve_handler_context.set(reserve_handler)
         provide_handler_context.set(self.provide_handler)
         #
         result = await self.assign(*args, **kwargs)
@@ -42,10 +40,9 @@ class FunctionalGenActor(FunctionalActor):
     async def assign(self,*args, **kwargs):
         raise NotImplementedError("This needs to be overwritten in order to work")
 
-    async def _assign(self, assign_handler: AssignHandler, reserve_handler: ReserveHandler, args, kwargs):
+    async def _assign(self, assign_handler: AssignHandler, args, kwargs):
 
         assign_handler_context.set(assign_handler)
-        reserve_handler_context.set(reserve_handler)
         provide_handler_context.set(self.provide_handler)
 
         try:
@@ -69,17 +66,17 @@ class FunctionalThreadedFuncActor(FunctionalActor):
         raise NotImplementedError("")   
 
 
-    def _assign_threaded(self, args, kwargs, loop, assign_handler, reserve_handler):
+    def _assign_threaded(self, args, kwargs, loop, assign_handler, provide_handler):
         assign_handler_context.set(assign_handler)
-        reserve_handler_context.set(reserve_handler)
+        provide_handler_context.set(provide_handler)
         loop_context.set(loop)
         res = self.assign(*args, **kwargs)
         loop_context.set(None)
         return res
 
-    async def _assign(self, assign_handler: AssignHandler, reserve_handler: ReserveHandler, args, kwargs):
+    async def _assign(self, assign_handler: AssignHandler, args, kwargs):
         try:
-            result = await self.loop.run_in_executor(self.threadpool, self._assign_threaded, args, kwargs, self.loop, assign_handler, reserve_handler)
+            result = await self.loop.run_in_executor(self.threadpool, self._assign_threaded, args, kwargs, self.loop, assign_handler, self.provide_handler)
             shrinked_returns = await shrinkOutputs(self.template.node, result) if self.shrinkOutputs else result
             await assign_handler.pass_result(shrinked_returns)
         except Exception as e:
