@@ -1,13 +1,12 @@
 import asyncio
 from bergen.registries.ward import get_ward_registry
-from bergen.wards.main import MainWard
+from bergen.wards.default import MainWard
 from bergen.config.types import ArkitektConfig, HerreConfig
 from bergen.schemas.herre.types import User
 from bergen.schema import Transcript
 from bergen.hookable.base import Hooks
 from pydantic.main import BaseModel
 from typing import Dict
-from bergen.wards.graphql.aiohttp import AIOHttpGraphQLWard
 from bergen.enums import ClientType, HostProtocol, PostmanProtocol, ProviderProtocol
 from bergen.logging import setLogging
 from bergen.auths.base import BaseAuthBackend
@@ -51,14 +50,8 @@ class BaseBergen:
             set_current_client(self)
 
 
-
         self.auth = auth
-
-
-
-        self.application = self.auth.application
-
-
+        self.auth.login()
 
         self.config = config
         self.client_type = client_type
@@ -68,6 +61,7 @@ class BaseBergen:
 
         self.host = config.host
         self.port = config.port
+        self.is_iternal = config.internal
         self.protocol = "https" if config.secure else "http"
 
         self._transcript = None
@@ -78,6 +72,13 @@ class BaseBergen:
         self._entertainer = None
         self.negotiate_additionals = {}
 
+        try:
+            self.loop = asyncio.get_event_loop()
+        except RuntimeError:
+            console.log("[yellow] Creating New EventLoop in This Thread")
+            self.loop = asyncio.new_event_loop()
+            asyncio.set_event_loop(self.loop)
+
         if auto_connect:
             self.negotiate()
 
@@ -159,8 +160,6 @@ class BaseBergen:
         return provider
     
     async def negotiate_async(self):
-        self.loop = asyncio.get_event_loop()
-
         from bergen.schemas.arkitekt.mutations.negotiate import NEGOTIATION_GQL
 
         # Instantiate our Main Ward, this is only for Nodes and Pods
@@ -168,9 +167,8 @@ class BaseBergen:
         await self.main_ward.configure()
 
         # We resort escalating to the different client Type protocols
-        self._transcript = await NEGOTIATION_GQL.run_async(ward=self.main_ward, variables={"clientType": self.client_type, **self.negotiate_additionals})
-        print(self._transcript)
-
+        self._transcript = await NEGOTIATION_GQL.run(ward=self.main_ward, variables={"clientType": self.client_type, "internal": self.is_iternal, **self.negotiate_additionals})
+ 
         #Lets create our different Wards 
         
         assert self._transcript.models is not None, "We apparently didnt't get any points"
@@ -178,6 +176,8 @@ class BaseBergen:
 
 
         ward_registry = get_ward_registry()
+
+        ward_registry.set_base(self.main_ward)
         
         for ward in self._transcript.wards:
             ward_registry.create_ward(self, ward)
@@ -265,9 +265,7 @@ class BaseBergen:
 
 
     def negotiate(self):
-        self.loop = asyncio.get_event_loop()
         assert not self.loop.is_running(), "You cannot negotiate with an already running Event loop, please ue negotiate_async"
-
         self.loop.run_until_complete(self.negotiate_async())
 
 
