@@ -1,4 +1,6 @@
 from asyncio.futures import Future
+from bergen.messages.postman.reserve.reserve_transition import ReserveState
+from bergen import messages
 from bergen.contracts.exceptions import AssignmentException
 from bergen.registries.client import get_current_client
 from bergen.schema import Node, NodeType
@@ -15,12 +17,9 @@ import logging
 logger = logging.getLogger(__name__)
 
 
-
-
-
 class Reservation:
 
-    def __init__(self, node: Node, loop=None, monitor: Monitor = None, ignore_node_exceptions=False, bounced=None, **params) -> None:
+    def __init__(self, node: Node, provision: str = None, loop=None, monitor: Monitor = None, ignore_node_exceptions=False, enter_on=[ReserveState.ACTIVE], exit_on=[ReserveState.ERROR, ReserveState.CANCELLED], bounced=None, **params) -> None:
         bergen = get_current_client()
         self._postman = bergen.getPostman()
 
@@ -28,8 +27,13 @@ class Reservation:
         self.params = ReserveParams(**params)
         self.monitor = monitor or current_monitor.get()
         self.ignore_node_exceptions = ignore_node_exceptions
+        self.state_hook = None
+        self.exit_states = exit_on
+        self.enter_states = enter_on
+        self.provision = provision
 
         self.bounced = bounced # with_bounced allows us forward bounced checks
+
         if self.bounced:
             assert "can_forward_bounce" in bergen.auth.scopes, "In order to use with_bounced forwarding you need to have the can_forward_bounced scope"
 
@@ -76,14 +80,15 @@ class Reservation:
         return Panel(columns, title="Reservation")
 
 
-    async def assign_async(self, *args, bypass_shrink=False, bypass_expand=False, **kwargs):
+    async def assign_async(self, *args, bypass_shrink=False, bypass_expand=False, persist=True, **kwargs):
         assert self.node.type == NodeType.FUNCTION, "You cannot assign to a Generator Node, use the stream Method!"
+        self.log("[green]ASSIGN",f"Assigning!")
         if self.critical_error is not None:
             
             self.log("[red]ASSIGN",f"Contract is broken and we can not assign. Exiting!")
         try:
             shrinked_args, shrinked_kwargs = await shrinkInputs(self.node, args, kwargs) if not bypass_shrink else (args, kwargs)
-            return_message = await self._postman.assign(self.reservation, shrinked_args, shrinked_kwargs=shrinked_kwargs, on_progress=self.on_progress, bounced=self.bounced)
+            return_message = await self._postman.assign(self.reservation, shrinked_args, shrinked_kwargs=shrinked_kwargs, on_progress=self.on_progress, persist=persist, bounced=self.bounced)
             outs = await expandOutputs(self.node, return_message.data.returns) if not bypass_expand else return_message.data.returns
             return outs
 
@@ -98,19 +103,19 @@ class Reservation:
 
 
 
-    async def stream(self, *args, bypass_shrink=False, bypass_expand=False, **kwargs):
+    async def stream(self, *args, bypass_shrink=False, bypass_expand=False, persist=True, **kwargs):
         assert self.node.type == NodeType.GENERATOR, "You cannot stream a Function Node, use the assign Method!"
         if self.critical_error is not None:
             self.log("[red]ASSIGN",f"Contract is broken and we can not assign. Exiting!")
         try:
             shrinked_args, shrinked_kwargs = await shrinkInputs(self.node, args, kwargs) if not bypass_shrink else (args, kwargs) 
-            async for message in self._postman.assign_stream(self.reservation, shrinked_args, serialized_kwargs=shrinked_kwargs, with_progress=True, bounced=self.bounced):
+            async for message in self._postman.assign_stream(self.reservation, shrinked_args, serialized_kwargs=shrinked_kwargs, with_progress=True, persist=persist, bounced=self.bounced):
 
                 if isinstance(message, AssignYieldsMessage):
                     outs = await expandOutputs(self.node, message.data.returns) if not bypass_expand else message.data.returns
                     yield outs
 
-                if isinstance(message, AssignProgressMessage):
+                if isinstance(message, AssignLogMessage):
                     if self.on_progress: self.on_progress(message.data.message, message.data.level)
                 
                 if isinstance(message, AssignCriticalMessage):
@@ -134,22 +139,14 @@ class Reservation:
     async def contract_worker(self):
         self.running = True
         try:
-            async for message in self._postman.reserve_stream(node_id=self.node.id, params_dict=self.params.dict(), with_progress=True, bounced=self.bounced):
+            async for message in self._postman.reserve_stream(node_id=self.node.id, provision=self.provision, params_dict=self.params.dict(), with_progress=True, bounced=self.bounced):
                 # Before here because Reserve Critical is actually an ExceptionMessage
                 #TODO: Undo this
 
-                if isinstance(message, ReserveProgressMessage):
+                if isinstance(message, ReserveLogMessage):
                     self.log(f'[green]{message.data.level.value}', message.data.message)
 
-                if isinstance(message, ProvideProgressMessage):
-                    self.log(f'[magenta]{message.data.level.value}', message.data.message)
-
-                if isinstance(message, ReserveCriticalMessage):
-                    # Reserve Errors are Errors that happen during the Reservation
-                    self.log(f'[red]EXCEPTION', message.data.message)
-                    self.critical_error = message
-
-                if isinstance(message, ProvideCriticalMessage):
+                elif isinstance(message, ReserveCriticalMessage):
                     # Reserve Errors are Errors that happen during the Reservation
                     self.log(f'[red]EXCEPTION', message.data.message)
                     self.critical_error = message
@@ -159,7 +156,24 @@ class Reservation:
                     self.contract_started.set_exception(message.toException())
                     return
 
-                elif isinstance(message, ReserveDoneMessage):
+                elif isinstance(message, ReserveTransitionMessage):
+                    # Once we acquire a reserved resource our contract (the inner part of the context can start)
+                    state = message.data.state
+
+                    if self.state_hook: await self.state_hook(self, message.data.state)
+                    
+                    if state in self.exit_states:
+                        self.critical_error = f"Exited because Reservation Changed to State {state}: {message.data.message}"
+
+                    if state in self.enter_states:
+                        pass#self.contract_started.set_result(message.meta.reference)
+
+
+                #elif isinstance(message, ReserveDoneMessage):
+                    # Once we acquire a reserved resource our contract (the inner part of the context can start)
+                #    self.contract_started.set_result(message.meta.reference)
+
+                elif isinstance(message, ReserveActiveMessage):
                     # Once we acquire a reserved resource our contract (the inner part of the context can start)
                     self.contract_started.set_result(message.meta.reference)
         
@@ -175,7 +189,8 @@ class Reservation:
             return
 
 
-    async def start(self):
+    async def start(self, state_hook = None):
+        self.state_hook = state_hook
         return await self.__aenter__()
 
     async def end(self):
@@ -201,11 +216,11 @@ class Reservation:
             except Exception as e:
                 self.log(f"[red]CRITICAL", f"Exitigin with {str(e)}")
     
-    def assign(self, *args, bypass_shrink=False, bypass_expand=False, **kwargs):
+    def assign(self, *args, bypass_shrink=False, bypass_expand=False, persist=True, **kwargs):
         if self.in_sync:
-            return self.loop.run_until_complete(self.assign_async(*args, bypass_shrink=False, bypass_expand=False, **kwargs))
+            return self.loop.run_until_complete(self.assign_async(*args, bypass_shrink=False, bypass_expand=False, persist=persist, **kwargs))
         else:
-            return self.assign_async(*args, bypass_shrink=False, bypass_expand=False, **kwargs)
+            return self.assign_async(*args, bypass_shrink=False, bypass_expand=False, persist=persist, **kwargs)
 
     def __enter__(self):
         self.in_sync = True
