@@ -1,6 +1,6 @@
 from abc import ABC, abstractmethod
 from asyncio.futures import Future
-from bergen.messages.postman.progress import ProgressLevel
+from bergen.messages.postman.log import LogLevel
 from bergen.messages import *
 from bergen.messages.base import MessageModel
 from bergen.postmans.utils import build_assign_message, build_reserve_message, build_unassign_messsage, build_unreserve_messsage
@@ -70,6 +70,7 @@ class BasePostman(Hookable):
     async def on_message(self, message: MessageModel):
         # First we check the streams
         reference = message.meta.reference
+        print(f"Incoming {message.meta.type}")
 
         if reference in self.assignment_stream_queues:
             return await self.assignment_stream_queues[reference].put(message)
@@ -86,9 +87,9 @@ class BasePostman(Hookable):
             else:
                 if reference in self.assignment_progress_functions:
                     function = self.assignment_progress_functions[reference]
-                    function(f"[red]Race Condition! Assignation Return before being Cancelled on the Server. Omitting Result!", level=ProgressLevel.DEBUG)
+                    function(f"[red]Race Condition! Assignation Return before being Cancelled on the Server. Omitting Result!", level=LogLevel.DEBUG)
 
-        elif isinstance(message, AssignProgressMessage):
+        elif isinstance(message, AssignLogMessage):
             if reference in self.assignment_progress_functions:
                 function = self.assignment_progress_functions[reference]
                 function(message.data.message, level=message.data.level)
@@ -99,13 +100,17 @@ class BasePostman(Hookable):
             future = self.assignments.pop(reference)
             future.set_exception(NodeException(message.data.message))
 
+        elif isinstance(message, AssignCancelledMessage):
+            console.log(message)
+            future = self.assignments.pop(reference)
+            
 
         # Unassign Function related
         elif isinstance(message, UnassignDoneMessage):
             future = self.unassignments.pop(reference)
             future.set_result(message)
 
-        elif isinstance(message, UnassignProgressMessage):
+        elif isinstance(message, UnassignLogMessage):
             if reference in self.unassignment_progress_functions:
                 function = self.unassignment_progress_functions[reference]
                 function(message.data.message, level=message.data.level)
@@ -122,7 +127,7 @@ class BasePostman(Hookable):
             future = self.reservations.pop(reference)
             future.set_result(message)
 
-        elif isinstance(message, ReserveProgressMessage):
+        elif isinstance(message, ReserveLogMessage):
             if reference in self.reservations_progress_functions:
                 function = self.reservations_progress_functions[reference]
                 function(message.data.message, level=message.data.level)
@@ -139,7 +144,7 @@ class BasePostman(Hookable):
             future = self.unreservations.pop(reference)
             future.set_result(message)
 
-        elif isinstance(message, UnreserveProgressMessage):
+        elif isinstance(message, UnreserveLogMessage):
             if reference in self.unreservations_progress_functions:
                 function = self.unreservations_progress_functions[reference]
                 function(message.data.message, level=message.data.level)
@@ -204,7 +209,7 @@ class BasePostman(Hookable):
 
 
 
-    async def assign(self, reservation: str, shrinked_args, shrinked_kwargs = {}, on_progress: Callable = None, bounced = None) -> AssignReturnMessage:
+    async def assign(self, reservation: str, shrinked_args, shrinked_kwargs = {}, on_progress: Callable = None, persist=True, bounced = None) -> AssignReturnMessage:
         """Assign takes a reservation and the serialized arguments as well as kwargs and awaits the result call, it will return
         the Assignation Returns.
 
@@ -235,7 +240,7 @@ class BasePostman(Hookable):
             self.assignment_progress_functions[assign_reference] = on_progress
             with_progress = True
             
-        assign = build_assign_message(assign_reference, reservation, shrinked_args, kwargs=shrinked_kwargs, with_progress=with_progress, bounced=bounced)
+        assign = build_assign_message(assign_reference, reservation, shrinked_args, kwargs=shrinked_kwargs, with_progress=with_progress, bounced=bounced, persist=persist)
         await self.forward(assign)
 
         try:
@@ -244,7 +249,7 @@ class BasePostman(Hookable):
 
 
         except asyncio.CancelledError as e:
-            if on_progress: on_progress(f"[red]Cancelled Assignment {assign_reference}", level=ProgressLevel.DEBUG)
+            if on_progress: on_progress(f"[red]Cancelled Assignment {assign_reference}", level=LogLevel.DEBUG)
             try:
                 unassign = await self.unassign(assign_reference, bounced=bounced)
             except:
@@ -273,7 +278,7 @@ class BasePostman(Hookable):
             return await future
 
         except asyncio.CancelledError as e:
-            if on_progress: on_progress(f"[red]Cancelled Unreservation {unreserve_reference}", level=ProgressLevel.DEBUG)
+            if on_progress: on_progress(f"[red]Cancelled Unreservation {unreserve_reference}", level=LogLevel.DEBUG)
             raise e
 
 
@@ -318,13 +323,14 @@ class BasePostman(Hookable):
             unassign = await self.unreserve(self, reserve_reference, bounced=bounced)
             raise e
 
-    async def reserve_stream(self, node_id: str = None, template_id: str = None , params_dict: dict = {}, with_progress= True, bounced=None) -> MessageModel:
+    async def reserve_stream(self, node_id: str = None, template_id: str = None , provision: str = None, params_dict: dict = {}, with_progress= True, bounced=None) -> MessageModel:
         reserve_reference = str(uuid.uuid4())
         self.reservation_stream_queues[reserve_reference] = asyncio.Queue()
 
 
-        reserve = build_reserve_message(reserve_reference, node_id, template_id, params_dict=params_dict, with_progress=with_progress, bounced=bounced)
+        reserve = build_reserve_message(reserve_reference, node_id, template_id, provision, params_dict=params_dict, with_progress=with_progress, bounced=bounced)
         await self.forward(reserve)
+        print("GOING OUT", reserve)
 
         try:
             while True:
@@ -338,11 +344,11 @@ class BasePostman(Hookable):
             raise e # Otherwise we
 
 
-    async def assign_stream(self, reservation: str, serialized_args, serialized_kwargs = {}, with_progress = None, bounced=None) -> MessageModel:
+    async def assign_stream(self, reservation: str, serialized_args, serialized_kwargs = {}, with_progress = None, persist=True, bounced=None) -> MessageModel:
         assign_reference = str(uuid.uuid4())
         self.assignment_stream_queues[assign_reference] = asyncio.Queue()
 
-        assign = build_assign_message(assign_reference, reservation, serialized_args, kwargs=serialized_kwargs, with_progress=with_progress, bounced=bounced)
+        assign = build_assign_message(assign_reference, reservation, serialized_args, kwargs=serialized_kwargs, with_progress=with_progress, bounced=bounced, persist=persist)
         await self.forward(assign)
 
         try:
