@@ -1,4 +1,5 @@
 from abc import abstractmethod
+from bergen.messages.postman.log import LogLevel
 from bergen.clients.base import BaseBergen
 from bergen.messages.base import MessageModel
 from bergen.hookable.base import Hookable, hookable
@@ -55,8 +56,12 @@ class BaseProvider(Hookable):
 
         self.template_actorClass_map = {}
 
+        self.message_queue = asyncio.Queue()
         self.provisions = {}
-        self.auto_provided_templates = [] # Templates that this provide will try to provide on startup
+        
+        
+    def log(self, message):
+        console.log(f"[blue] {message}")
 
 
     def template(self, node: Node, policy: PodPolicy = MultiplePodPolicy(), auto_provide=False, on_provide=None,  on_unprovide=None, bypass_shrink=False, bypass_expand=False, **implementation_details):
@@ -113,10 +118,10 @@ class BaseProvider(Hookable):
             assert callable(function_or_actor) or issubclass(function_or_actor, Actor), "Please only decorate functions or subclasses of Actor"
 
             if isactor(function_or_actor):
-                console.log("Already is Actor. Creating Node")
+                self.log("Already is Actor. Creating Node")
                 node = createNodeFromActor(function_or_actor, allow_empty_doc=allow_empty_doc, widgets=widgets)
             else:
-                console.log("Is Function. Creating Node and Wrapping")
+                self.log("Is Function. Creating Node and Wrapping")
                 node = createNodeFromFunction(function_or_actor, allow_empty_doc=allow_empty_doc, widgets=widgets)
 
             # We pass this down to our truly template wrapper that takes the node and transforms it
@@ -142,30 +147,21 @@ class BaseProvider(Hookable):
 
 
     async def on_message(self, message: MessageModel):
-        if isinstance(message, BouncedProvideMessage):
-            logger.info("Received Provide Request")
-            assert message.data.template is not None, "Received Provision that had no Template???"
-            await self.handle_bounced_provide(message)
-
-        elif isinstance(message, BouncedUnprovideMessage):
-            logger.info("Received Unprovide Request")
-            assert message.data.provision is not None, "Received Unprovision that had no Provision???"
-            await self.handle_bounced_unprovide(message)
-
-        else: 
-            raise Exception("Received Unknown Task")
+        await self.message_queue.put(message)
 
 
     async def handle_bounced_provide(self, message: BouncedProvideMessage):
         try:
             await self.on_bounced_provide(message)
 
-            progress = ProvideProgressMessage(data={
-            "level": "INFO",
+            progress = ProvideLogMessage(data={
+            "level": LogLevel.INFO,
             "message": f"Pod Pending"
             }, meta={"extensions": message.meta.extensions, "reference": message.meta.reference})
 
             await self.forward(progress)
+
+
         except Exception as e:
             logger.error(e)
             critical_error = ProvideCriticalMessage(data={
@@ -175,21 +171,25 @@ class BaseProvider(Hookable):
             await self.forward(critical_error)
             raise e
 
-    async def handle_bounced_unprovide(self, message: BouncedProvideMessage):
+
+    async def handle_bounced_unprovide(self, message: BouncedUnprovideMessage):
         try:
             await self.on_bounced_unprovide(message)
 
-            progress = UnprovideProgressMessage(data={
-            "level": "INFO",
-            "message": f"Pod Unproviding"
+            progress = UnprovideLogMessage(data={
+            "level": LogLevel.INFO,
+            "message": f"Pod Unproviding",
+            "provision": message.data.provision,
             }, meta={"extensions": message.meta.extensions, "reference": message.meta.reference})
 
             await self.forward(progress)
 
         except Exception as e:
-            logger.error(e)
+            logger.exception(e)
             critical_error = UnprovideCriticalMessage(data={
-            "message": str(e)
+            "message": str(e),
+            "type": str(e.__class__.__name__),
+            "provision": message.data.provision,
             }, meta={"extensions": message.meta.extensions, "reference": message.meta.reference})
 
             await self.forward(critical_error)
@@ -198,33 +198,50 @@ class BaseProvider(Hookable):
     @hookable("bounced_provide", overwritable=True)
     async def on_bounced_provide(self, message: BouncedProvideMessage):
         actorClass = await self.get_actorclass_for_template(message.data.template)
-        console.log(f"[red]Got provision request for {message.data.template} and will entertain {actorClass.__name__}")
+        self.log(f"Got provision request for {message.data.template} and will entertain {actorClass.__name__}")
         await self.client.entertainer.entertain(message, actorClass) # Run in parallel
 
     @hookable("bounced_unprovide", overwritable=True)
     async def on_bounced_unprovide(self, message: BouncedUnprovideMessage):
-        console.log(f"[red]Got unprovision. Sending to entertainer")
+        self.log(f"Got unprovision. Sending to entertainer")
         await self.client.entertainer.unentertain(message)
 
     @hookable("get_actorclass_for_template", overwritable=True)
     async def get_actorclass_for_template(self, template_id):
-        assert template_id in self.template_actorClass_map, f"We have no Actor stored in our list {template_id}"
+        assert template_id in self.template_actorClass_map, f"We have no Actor stored in our list for Template with ID {template_id}"
         return self.template_actorClass_map[template_id]
 
     def register_actor(self, template_id: str, actorClass: Type[Actor]):
         assert template_id not in self.template_actorClass_map, f"We cannot register two Actors for the same template {template_id}"
         self.template_actorClass_map[template_id] = actorClass
-        console.log(f"[red]Registered Actor {actorClass.__name__} for Template {template_id}")
+        self.log(f"Registered Actor {actorClass.__name__} for Template {template_id}")
 
     async def provide_async(self):
-        while True:
-            await asyncio.sleep(3)
 
+        self.log(" ------------- AWAITING MESSAGES -----------")
+        while True:
+            message = await self.message_queue.get()
+            
+            if isinstance(message, BouncedProvideMessage):
+                logger.info("Received Provide Request")
+                assert message.data.template is not None, "Received Provision that had no Template???"
+                await self.handle_bounced_provide(message)
+
+            elif isinstance(message, BouncedUnprovideMessage):
+                logger.info("Received Unprovide Request")
+                assert message.data.provision is not None, "Received Unprovision that had no Provision???"
+                await self.handle_bounced_unprovide(message)
+            else: 
+                self.log(f"Provider: [red] Received unknown message {message}")
+
+            self.message_queue.task_done()
+
+            
     def provide(self):
         if self.loop.is_running():
-            logger.error("Cannot do this, please await run()")
+            raise Exception("Cannot do this in a running loop, please await or create task of provide_async()")
         else:
-            self.loop.run_forever()
+            self.loop.run_until_complete(self.provide_async())
 
         # we enter a never-ending loop that waits for data
         # and runs callbacks whenever necessary.
