@@ -1,6 +1,7 @@
 from abc import abstractmethod
 from abc import ABC
 import asyncio
+from bergen.auths.base import BaseAuthBackend
 from bergen.schema import DataPoint, WardSettings
 from bergen.query import  TypedGQL
 from typing import TypeVar
@@ -12,31 +13,75 @@ T = TypeVar("T")
 class WardException(Exception):
     pass
 
+class GraphQLException(WardException):
+    pass
+
 class ConnectionError(Exception):
     pass
 
-
+class TokenExpired(ConnectionError):
+    pass
 
 
 class BaseWard(ABC):
 
-    def __init__(self, client, settings: WardSettings, loop=None):
+    def __init__(self, client, loop=None):
         self.loop = loop or client.loop or asyncio.get_event_loop()
+        self.auth: BaseAuthBackend = client.auth
+        assert self.auth.access_token is not None, "Cannot create a Ward without acquiring a Token first"
+        self._headers = {"Authorization": f"Bearer {self.auth.access_token}"}
 
+    @abstractmethod
+    async def connect(self):
+        """Everytime we need to reastablish a connection because of a Token Refersh"""
+        pass
+
+    @abstractmethod
+    async def configure(self):
+        """The initial connection attempt (also negotiation can happen here)"""
+        pass
+
+
+    @abstractmethod
+    async def disconnect(self):
+        """CleanUp"""
+        pass
+
+
+    async def run(self, gql: TypedGQL, variables: dict = {}):
+        print("Called")
+        try:
+            return await self.pass_async(gql, variables =variables)
+
+        except TokenExpired:
+            console.print_exception()
+
+            self.auth.refetch()
+            self._headers = {"Authorization": f"Bearer {self.auth.access_token}"}
+
+            await self.disconnect()
+            await self.connect()
+
+            return await self.pass_async(gql, variables=variables)
+
+
+
+    @abstractmethod
+    def pass_async(self, gql: TypedGQL, variables: dict = {}):
+        return gql.cls(**{})
+
+
+
+
+class ServiceWard(BaseWard):
+
+    def __init__(self, client, settings: WardSettings, loop=None):
         self.distinct = settings.distinct
         self.needs_negotiation = settings.needsNegotiation
         self.host = settings.host or client.config.host
         self.port = settings.port or client.config.port
         self.protocol = "https" if settings.secure or client.config.secure else "http"
-
-        self.token = client.auth.access_token
-        assert self.token is not None, "Cannot create a Ward without acquiring a Token first"
-
-
-    @abstractmethod
-    async def connect(self):
-        pass
-
+        super().__init__(client, loop=loop)
 
     async def configure(self):
         try:
@@ -47,28 +92,20 @@ class BaseWard(ABC):
         except:
             console.print_exception()
             raise ConnectionError(f"Ward {self.distinct}: Connection to {self.host}:{self.port} on {self.port} Failed")
-        
 
 
-    def run(self, the_query: TypedGQL, variables: dict = {}, **kwargs):
-        return self.loop.run_until_complete(self.run_async(the_query, variables=variables, **kwargs))
 
+class MainWard(BaseWard):
 
-    @abstractmethod
-    def run_async(self, gql: TypedGQL, variables: dict = {}):
-        return gql.cls(**{})
-
-
-    @abstractmethod
-    async def disconnect(self):
-        pass
-
-
-    async def __aenter__(self):
-        await self.configure()
-        return self
-
-
-    async def __aexit__(self, *args, **kwargs):
-        await self.disconnect()
+    def __init__(self, client, loop=None):
+        self.host = client.config.host
+        self.port = client.config.port
+        self.protocol = "https" if client.config.secure else "http"
+        super().__init__(client, loop=loop)
 
+    async def configure(self):
+        try:
+            await self.connect()
+        except:
+            console.print_exception()
+            raise ConnectionError(f"BaseWard: Connection to {self.host}:{self.port} on {self.port} Failed")
